function [G d u s p]=mpccp(X,lambda,maxiter,gamma,maxr,initer)
% function G=mpccp(X,lambda,maxiter,gamma);
%
% This function produces a fast approximation of the solution to
% \min_{G,d} \|G\|_* + \lambda \| d \|_1, s.t., X = G + d
% The iterative algorithm can be obtained by ADMM (alternating direction
% method of multipliers) or the split Bregman.
% The singular value thresholding is carried out for the first maxr values;
% rest are set to zero. When maxr = inf, the solution is accurate.
% The SVD (singular value decomposition) is obtained by npisvd
%
% maxr is the guess at the maximum possible rank.  
% this code is fast only if maxr is smaller than the smaller dimension of X
%  
% Input:
%      X            -     A mxn matrix
%      lambda       -     The model parameter as in the above objective function 
%      maxiter      -     Maximum number of iterations.
%      gamma        -     The extra parameter appears in augmented
%                         Lagrangian method.
%      maxr         -     The guessed rank of X.
%      tol          -     Tolerance value.
%      initer       -     The number of qr iterations when computing svds. 1 or 2 is %often enough, and 20 is usually more than enough.
%
% Output:
%      G ,d         -     mxn matrices
%                         G + d = X; G is nearly low-rank and d sparse. 

[m n]=size(X);
%d=0;
d=X;
B=0;
G=0;
debug=0;
if exist('maxr');
    if maxr~=inf
        p=randn(n,maxr);
    end
%    p=randn(m,maxr);
else
    maxr=inf;
end

for it=1:maxiter
    % Solve for the sparse part
    % min_d \lambda \| d \|_1 + \gamma/2 \| X - G - d + B/\gamma \|_F^2
    d=shrink(X-G+B/gamma,lambda/gamma);
    % Solve for the low-rank part
    % min_G \| G \|_* + \gamma/2 \| X - G - d + B/\gamma \|_F^2
    if maxr==inf
        G=svdshrink(X-d+B/gamma,1/gamma);
    else
        [G p u s]=svdshrinkqr(X-d+B/gamma,p,1/gamma,maxr,initer);
    end
    % Update the dual variable
    B=B+gamma*(X-G-d);
    if debug
        if mod(it,10)==1
            err((it-1)/10+1)=norm(X-G-d,'fro');
            figure(5);plot(err);
        end
    end
end

if maxr==inf
    [u s p]=svd(G);
end

end







function [L U]=svdshrink(X,gamma);
[U S V]=svd(X,'econ');
s=diag(S);
s=shrink(s,gamma);
s=diag(s);
L=U*(s*V');
end

function [L v u s]=svdshrinkqr(X,p,gamma,maxr,initer);
     for kk=1:initer
         [q r]=qr(X*p,0);
         q=q';
         [p r]=qr((q*X)',0);
     end
     [ut,st,vt] = svd(X*p,'econ');

     v=p*vt;
     s=st(1:maxr,1:maxr);
     v=v(:,1:maxr);
     u=ut(:,1:maxr);
    s=diag(s);
    s=shrink(s,gamma);
    s=diag(s);
    L=u*(s*v');
end

function X=shrink(Y,mu);
%function X=shrink(Y,mu);
X=(Y-mu*sign((Y))).*(abs(Y)>=mu);
end
