function [G d dif dif1 J status]=mpccp_relax(X,lambda,maxiter,gamma,maxr,initer,tol1)
% function G=mpccp_relax(X,lambda,maxiter,gamma);
%
% This function produces a fast approximation of the solution to
% \min_{G,d} \|G\|_* + \lambda \| d \|_1 + \gamma/2 \| X - G - d \|_F^2
% _relax is the version to bear Gaussian noise
%
% The iterative algorithm can be obtained by alternating direction method.
% The singular value thresholding is carried out for the first maxr values;
% rest are set to zero. When maxr = inf, the solution is accurate.
% The SVD (singular value decomposition) is obtained by npisvd
% maxr is the guess at the maximum possible rank.  this code is fast only 
% if maxr is smaller than the smaller dimension of X
% initer is the number of qr iterations when computing svds.  1 or 2 is 
% often enough, and 20 is usually more than enough.
%
% Input:
%      X            -     A mxn matrix
%      lambda       -     The model parameter as in the above objective function 
%      maxiter      -     Maximum number of iterations.
%      gamma        -     The model parameter as in the above objective function
%      maxr         -     The guessed rank of X.
%      tol1          -     Tolerance value.
%      initer       -     The number of qr iterations when computing svds. 1 or 2 is %often enough, and 20 is usually more than enough.
%
% Output:
%      G ,d         -     mxn matrices
%                         G + d is close to X; G is nearly low-rank and d sparse. 

if nargin < 6
    initer = 2;
end
[~,n]=size(X);

d=X;

G=0;
debug=0;
if exist('maxr','var');
    if maxr~=inf
        p=randn(n,maxr);
    end
else
    maxr=inf;
end


if exist('tol1','var')==0
    tol1 = 1e-4;
end

it = 1;
dif(1) = Inf;
J(1) = Inf;
dif1(1) = Inf;
while it<=maxiter && dif1(it)>1+tol1 || dif1(it)<1-tol1  
    G_old = G;
    % Solve for the sparse part
    % min_d \lambda \| d \|_1 + \gamma/2 \| X - G - d \|_F^2
    d=shrink(X-G,lambda/gamma);
    % Solve for the low-rank part
    % min_G \| G \|_* + \gamma/2 \| X - G - d \|_F^2
    if maxr==inf
        G=svdshrink(X-d,1/gamma);
    else
        [G p u s]=svdshrinkqr(X-d,p,1/gamma,maxr,initer);
    end
    if debug
        if mod(it,10)==1
            err((it-1)/10+1)=norm(X-G-d,'fro');
            figure(5);plot(err);
        end
    end
    
    it = it+1;
    dif(it) = norm(G-G_old,'fro')/norm(G_old,'fro');
    [u s v] = svd(G);
    J(it) = sum(diag(s)) + lambda*sum(sum(abs(d))) + .5*gamma*norm(X-G-d,'fro')^2;
    dif1(it) = J(it) / J(it-1);
    
end

status = 1;
if it > maxiter
    status = 0;
end


if maxr==inf
    [u s p]=svd(G);
end

end







function [L U]=svdshrink(X,gamma);
[U S V]=svd(X,'econ');
s=diag(S);
s=shrink(s,gamma);
s=diag(s);
L=U*(s*V');
end

function [L v u s]=svdshrinkqr(X,p,gamma,maxr,initer);
for kk=1:initer
    [q r]=qr(X*p,0);
    q=q';
    [p r]=qr((q*X)',0);
end
[ut,st,vt] = svd(X*p,'econ');

v=p*vt;
s=st(1:maxr,1:maxr);
v=v(:,1:maxr);
u=ut(:,1:maxr);
s=diag(s);
s=shrink(s,gamma);
s=diag(s);
L=u*(s*v');
end

function X=shrink(Y,mu);
X=(Y-mu*sign((Y))).*(abs(Y)>=mu);
end
