function [G d]=mpccppro(X,lambda,maxiter,gamma,maxr,tol);
% function [G d]=mpccppro(X,lambda,maxiter,gamma,maxr,tol);
%
% This function produces a fast approximation of the solution to
% \min_{G,d} \|G\|_* + \lambda \| d \|_1, s.t., X = G + d
% The iterative algorithm can be obtained by ADMM (alternating direction
% method of multipliers) or the split Bregman.
% The singular value thresholding is carried out for the first maxr values;
% rest are set to zero. When maxr = inf, the solution is accurate.
% The SVD (singular value decomposition) is obtained by propack instead of npisvd
%
% Input:
%      X            -     A mxn matrix
%      lambda       -     The model parameter as in the above objective function 
%      maxiter      -     Maximum number of iterations.
%      gamma        -     The extra parameter appears in augmented
%                         Lagrangian method.
%      maxr         -     The guessed rank of X.
%      tol          -     Tolerance value.
%
%
% Output:
%      G ,d         -     mxn matrices
%                         G + d = X; G is nearly low-rank and d sparse. 

debug=0;
[m n]=size(X);
%d=0;
d=X;
B=0;
G=0;

if exist('maxr')==0;
    maxr=inf;
end
if exist('tol')==0;
     opts.tol=16*eps;
else
    opts.tol=tol;
end


for it=1:maxiter
    % Solve for the sparse part
    % min_d \lambda \| d \|_1 + \gamma/2 \| X - G - d + B/\gamma \|_F^2
    d=shrink(X-G+B/gamma,lambda/gamma);
    % Solve for the low-rank part
    % min_G \| G \|_* + \gamma/2 \| X - G - d + B/\gamma \|_F^2
    if debug
        [G time(it)] = svdshrink(X-d+B/gamma,1/gamma,maxr,opts,debug);
    else
        G = svdshrink(X-d+B/gamma,1/gamma,maxr,opts,debug);
    end
    % Update the dual variable
    B=B+gamma*(X-G-d);
    if debug
        if mod(it,10)==1
            err((it-1)/10+1)=norm(X-G-d,'fro');
            figure(5);plot(err);
            time
        end
    end
end


end



function [L time]=svdshrink(X,gamma,maxr,opts,debug);
if maxr==inf
    [U S V]=svd(X,'econ');
else
    if debug
    tic;
    end
    [U,S,V] = lansvd(X,maxr,'L',opts);
    if debug
    time=toc;
    end
end
s=diag(S);
s=shrink(s,gamma);
s=diag(s);
L=U*(s*V');
end

function X=shrink(Y,mu);
%function X=shrink(Y,mu);
X=(Y-mu*sign((Y))).*(abs(Y)>=mu);
end
