function [G d dif dif1 J status]=mpccppro_relax(X,lambda,maxiter,gamma,maxr,tol1,tol)
%function G=mpccppro(X,lambda,maxiter,gamma,maxr,tol);
%
% This function produces a fast approximation of the solution to
% \min_{G,d} \|G\|_* + \lambda \| d \|_1 + \gamma/2 \| X - G - d \|_F^2
% _relax is the version to bear Gaussian noise
%
% The iterative algorithm can be obtained by alternating direction method.
% The singular value thresholding is carried out for the first maxr values;
% rest are set to zero. When maxr = inf, the solution is accurate.
% The SVD (singular value decomposition) is obtained by propack instead of npisvd
%
% Input:
%      X            -     A mxn matrix
%      lambda       -     The model parameter as in the above objective function 
%      maxiter      -     Maximum number of iterations.
%      gamma        -     The model parameter as in the above objective function
%      maxr         -     The guessed rank of X.
%      tol          -     Tolerance value.
%
% Output:
%      G ,d         -     mxn matrices
%                         G + d is close to X; G is nearly low-rank and d sparse. 
debug=0;
[m n]=size(X);
%d=0;
d=X;
B=0;
G=0;

if exist('maxr','var')==0;
    maxr=inf;
end

if exist('tol','var')==0;
    opts.tol=16*eps;
else
    opts.tol=tol;
end

if exist('tol1','var')==0;
    tol1 = 1e-4;
end

it = 1;
dif(1) = Inf;
J(1) = Inf;
dif1(1) = Inf;
while it<=maxiter && dif1(it)>1+tol1 || dif1(it)<1-tol1  
    % Solve for the sparse part
    % min_d \lambda \| d \|_1 + \gamma/2 \| X - G - d \|_F^2
    d=shrink(X-G,lambda/gamma);
    G_old = G;
    % Solve for the low-rank part
    % min_G \| G \|_* + \gamma/2 \| X - G - d \|_F^2
    if debug
        [G s time(it)]=svdshrink(X-d+B/gamma,1/gamma,maxr,opts,debug);
    else
        [G s]=svdshrink(X-d,1/gamma,maxr,opts,debug);
    end
    
    if debug
        if mod(it,10)==1
            err((it-1)/10+1)=norm(X-G-d,'fro');
            figure(5);plot(err);
            time
        end
    end
    
    it = it+1;
    dif(it) = norm(G-G_old,'fro')/norm(G_old,'fro');
    J(it) = sum(diag(s)) + lambda*sum(sum(abs(d))) + .5*gamma*norm(X-G-d,'fro')^2;
    dif1(it) = J(it) / J(it-1);
    
end

status = 1;
if it > maxiter
    status = 0;
end

end



function [L s time]=svdshrink(X,gamma,maxr,opts,debug)
if maxr==inf
    [U S V]=svd(X,'econ');
else
    if debug
        tic;
    end
    [U,S,V] = lansvd(X,maxr,'L',opts);
    if debug
        time=toc;
    end
end
s=diag(S);
s=shrink(s,gamma);
s=diag(s);
L=U*(s*V');
end

function X=shrink(Y,mu)
X=(Y-mu*sign((Y))).*(abs(Y)>=mu);
end
